import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.rabbitmq.client.Delivery;

import org.apache.commons.io.IOUtils;
//import XML related packages
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class AMQPMessage {
	private SimpleDateFormat sdfTime = new SimpleDateFormat("HH:mm:ss.SSS") ;
	
	public String clientReceptionTime ="";
	public String AMQPSendingTime ="";
    public String xmlMessagePayload = "";
    public String amqpProperties = "";
    public String correlationId = "";
    public String type = "";
    public String last = "";
    public String routingKeyName = "";
    public String sequenceNo = "";
    private SimpleDateFormat sdfStatsWithDash = new SimpleDateFormat("yyy-MM-dd HH:mm:ss.SSS") ;
    private RequestResponseList request_responseList;
    private String response_OR_Broadcast;
    public String contentEncoding = ""; // GZIP compressed or not
    
    private static final String MESSAGE_ENCODING = "UTF-8";
	
    // constructor
    public AMQPMessage(RequestResponseList request_responseList, String response_OR_Broadcast ) {
    	this.request_responseList = request_responseList;
    	this.response_OR_Broadcast = response_OR_Broadcast;
    }
    
	// **********************************************************
	// *** processMessage
	// **********************************************************    	   
    public void processMessage (Delivery messageDelivery, Log appLog, String response_OR_Broadcast) {
    // decompress (gzip) the message if required  
    // write the properties and XML payload in the GUI and Log file
    // count the nb of order events when the broadcast is an Order Execution Report
    
    correlationId = messageDelivery.getProperties().getCorrelationId();
    	 
   // String decomp = "";
    if (messageDelivery != null) {
               
    	this.clientReceptionTime = sdfTime.format(new Date());
    	this.contentEncoding = messageDelivery.getProperties().getContentEncoding();
    	
    	if (contentEncoding == null )
          {
            // System.out.println("ContentEncoding <> gzip => Message NOT compressed");
             try {
            	if (response_OR_Broadcast.contentEquals("RESPONSE")) {
            		request_responseList.updateRequestWithResponseTime(correlationId, messageDelivery.getProperties().getType(), Globals.REQ_STATE_NOT_COMPRESSED);
            	}
            	 xmlMessagePayload = new String(messageDelivery.getBody(), MESSAGE_ENCODING);
            	// Globals.guiWindowDebug.AddText("not compressed: "+ xmlMessagePayload);
            	// decomp = "Properties = " + (messageDelivery.getProperties())+"\n====> XML = "+xmlMessagePayload;
            	
				} catch (UnsupportedEncodingException e) {
				//TODO Auto-generated catch block
				e.printStackTrace();
				}
          }
          else // zipped message
          {
             try {
            	if (messageDelivery.getProperties().getType() != null) {
	         			if (messageDelivery.getProperties().getType().equals("com.deutscheboerse.m7.trading.api.v6.FullOrdrCaptureResp")) {
	        				String msg = "FullOrdrCaptureResp msg reception: correlation Id ='"+  correlationId + "': unzipping start...";
	        				//Globals.guiWindowDebug.AddText(msg);
	        				appLog.logger.info(msg);
	        				System.out.println(msg);
	        			}
	         			if (response_OR_Broadcast.contentEquals("RESPONSE") && !correlationId.contains("LOAD")) {
	                		request_responseList.updateRequestWithResponseTime(correlationId, messageDelivery.getProperties().getType(), Globals.REQ_STATE_UNZIPPING);
	                	}
            	}
            	
            	 xmlMessagePayload = decompress(messageDelivery.getBody());
            	 
      			if (response_OR_Broadcast.contentEquals("RESPONSE")) {
            		request_responseList.updateRequestWithResponseTime(correlationId, messageDelivery.getProperties().getType(), Globals.REQ_STATE_UNZIPPED);
            	}
            	 //	decomp = "getProperties = " + (messageDelivery.getProperties())+"\n====> XML = " + xmlMessagePayload;
			} catch (IOException e) {
			//TODO Auto-generated catch block
			e.printStackTrace();
			}
         }
    	
       	// extract (for broadcasts only) gap info : Routing Key and sequence No
    	if (!messageDelivery.getEnvelope().getRoutingKey().contains("m7.private.responseQueue")) {
			String[] headerValue = messageDelivery.getProperties().getHeaders().toString().split(",");
		
			this.sequenceNo= headerValue[0].substring("{x-m7-group-sequence=".length());
			this.routingKeyName  = headerValue[1].substring(" x-m7-group-id=".length());
			//System.out.println(this.getClass().getName() + " " + routingKeyName + " : " + sequenceNo);
    	}
    }
           else {
               System.out.println("delivery is null");
           } 
    type = messageDelivery.getProperties().getType();
    amqpProperties = messageDelivery.getProperties().toString();

    if (messageDelivery.getProperties().getType() != null) {
        this.AMQPSendingTime = extractServerTimestamp(amqpProperties);    	
    }
    else this.AMQPSendingTime = "M7 heartbeat";

    System.out.println("this.AMQPSendingTime = " + this.AMQPSendingTime); 
    //correlationId = messageDelivery.getProperties().getCorrelationId();
    
   
   // guiWindow.AddText("------------------------------\nProcessed msg ="+ decomp+"------------------------------\n");
   // return decomp;
    System.out.println(this.type  +  " correlation Id = " + this.correlationId + " : end of decompression");
   // Globals.guiWindowDebug.AddText(response_OR_Broadcast+" : "+this.type);
    }

    // **********************************************************
	// *** Decompress gzip
	// ********************************************************** 
    public static String decompress(final byte[] compressed) throws IOException {
        ByteArrayInputStream bis = new ByteArrayInputStream(compressed);
        GZIPInputStream gis = new GZIPInputStream(bis);
        byte[] bytes = IOUtils.toByteArray(gis);
        return new String(bytes, "UTF-8");
    }

    // **********************************************************
	// *** Compress gzip
	// ********************************************************** 
    public static byte[] compress(final String str) throws IOException {
        if ((str == null) || (str.length() == 0)) {
          return null;
        }
        ByteArrayOutputStream obj = new ByteArrayOutputStream();
        GZIPOutputStream gzip = new GZIPOutputStream(obj);
        gzip.write(str.getBytes("UTF-8"));
        gzip.flush();
        gzip.close();
        return obj.toByteArray();
      }
    
    // **********************************************************
	// *** extract from AMQP properties the content of the server-timestamp
	// **********************************************************
    public String extractServerTimestamp(String s) {

    	String prefix = "server-timestamp=";
    	String messageServerTimestamp_String ;
    	
    	s = s.substring(s.indexOf(prefix) + prefix.length());
    	String[] result = s.split("}");

    	try {
    		long messageServerTimestamp_Long = Long.parseLong(result[0]);
    		messageServerTimestamp_String = sdfStatsWithDash.format(new Date(messageServerTimestamp_Long));
    	}
    	catch (Exception e) {
    		messageServerTimestamp_String = "N/A";
    	}
    	    	
    	return messageServerTimestamp_String;
    }
	/*
	 * // ********************************************************** // ***
	 * parseOrderExeReportAndUpdateOrderList //
	 * ********************************************************** public void
	 * parseOrderExeReportAndUpdateOrderList(List<Order> orderList,
	 * List<OrderBeingProcessed> orderBeingProcessedByM7List, Log appLog, GUIWindow
	 * guiWindow) throws IOException { String orderExeReportOrderId = ""; String
	 * orderExeReportClientOrderId = ""; String orderExeReportRevisionNo = "";
	 * String orderExeReportState = ""; String orderExeReportAction = "";
	 * 
	 * 
	 * guiWindow.AddText("start of parseOrderExeReportAndUpdateOrderList");
	 * displayOrderList(orderList, guiWindow) ;
	 * 
	 * DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
	 * DocumentBuilder builder; try { builder = docFactory.newDocumentBuilder();
	 * 
	 * // Create a Document from a file or stream StringBuilder xmlStringBuilder =
	 * new StringBuilder(); xmlStringBuilder.append(this.xmlMessagePayload);
	 * 
	 * ByteArrayInputStream input = new
	 * ByteArrayInputStream(xmlStringBuilder.toString().getBytes("UTF-8")); Document
	 * document = null; try { document = builder.parse(input);
	 * document.getDocumentElement().normalize();
	 * 
	 * // Extract the root element // Element root = document.getDocumentElement();
	 * 
	 * NodeList nodeList = document.getElementsByTagName("Ordr");
	 * 
	 * boolean recordFound = false; // go through each order record and check if the
	 * client Order ID matches one in the Order List : if so update the revision No
	 * and m7 order ID for (int itr = 0; itr < nodeList.getLength(); itr++) { if
	 * (recordFound == true) break; Node node = nodeList.item(itr); //
	 * guiWindow.AddText("Node Name :" + node.getNodeName()); //
	 * guiWindow.AddText("node type type =" + node.getNodeType()); if
	 * (node.getNodeType() == Node.ELEMENT_NODE) { Element eElement = (Element)
	 * node;
	 * 
	 * NamedNodeMap nodeMap = eElement.getAttributes();
	 * 
	 * // go through each attribute of the current Element and search for
	 * "Client Order Id" for (int i = 0; i < nodeMap.getLength(); i++) { Node node2
	 * = nodeMap.item(i); String nodeName =node2.getNodeName(); String nodeValue
	 * =node2.getNodeValue();
	 * 
	 * guiWindow.AddText("attr name : " + nodeName + " | value : " + nodeValue);
	 * 
	 * // store temporarily the ordrId, clOrdrId and revisionNo
	 * 
	 * // if reaching the attribute to retrieve, store it temporarily if
	 * (nodeName.contentEquals("ordrId") ) { orderExeReportOrderId = nodeValue;
	 * guiWindow.AddText("Order Id found : "+ " OrderId = " + orderExeReportOrderId
	 * ); }
	 * 
	 * if (nodeName.contentEquals("clOrdrId") ) { orderExeReportClientOrderId =
	 * nodeValue; guiWindow.AddText("Client Order Id found : "+ " ClientOrderId= " +
	 * orderExeReportClientOrderId ); }
	 * 
	 * if (nodeName.contentEquals("revisionNo") ) { orderExeReportRevisionNo =
	 * nodeValue; guiWindow.AddText("RevisionNo found : "+" RevisionNo= "+
	 * orderExeReportRevisionNo); }
	 * 
	 * if (nodeName.contentEquals("state") ) { orderExeReportState = nodeValue;
	 * guiWindow.AddText("State found : "+" RevisionNo= "+ orderExeReportState); }
	 * if (nodeName.contentEquals("action") ) { orderExeReportAction = nodeValue;
	 * guiWindow.AddText("Action found : "+" Action= "+ orderExeReportAction); }
	 * 
	 * } // once all attributes have been parsed : // check if the Client Order Id
	 * value exists in the Order List and add the revision No and order ID if found
	 * 
	 * guiWindow.AddText("End of record parsing :3 attributes found : "+
	 * " OrderId = " + orderExeReportOrderId + " ClientOrderId= " +
	 * orderExeReportClientOrderId + " RevisionNo= "+ orderExeReportRevisionNo);
	 * updateOrderList(orderList, orderExeReportOrderId,
	 * orderExeReportClientOrderId, orderExeReportRevisionNo, orderExeReportState,
	 * orderExeReportAction, guiWindow);
	 * guiWindow.AddText("back to parseOrderExeReportAndUpdateOrderList");
	 * 
	 * trackOrderExeReportProcessing(orderBeingProcessedByM7List,
	 * orderExeReportClientOrderId,guiWindow);
	 * 
	 * 
	 * } // if (node.getNodeType() == Node.ELEMENT_NODE) } // // go through each
	 * order record
	 * 
	 * } catch (SAXException e) { // TODO Auto-generated catch block
	 * e.printStackTrace(); }
	 * 
	 * } catch (ParserConfigurationException e) { // TODO Auto-generated catch block
	 * e.printStackTrace(); }
	 * 
	 * } // ********************************************************** // ***
	 * trackOrderExeReportProcessing //
	 * ********************************************************** public void
	 * trackOrderExeReportProcessing(List<OrderBeingProcessed>
	 * orderBeingProcessedList,String fileOrderIdToBeRemoved, GUIWindow guiWindow) {
	 * 
	 * guiWindow.
	 * AddText("-----------Order Exe Reception: tag an order as confirmed by M7 / Order Exe Report received"
	 * ); displayOrderBeingProcessedList(orderBeingProcessedList, guiWindow);
	 * 
	 * // find the 1st occurrence of the order to be removed for
	 * (OrderBeingProcessed next : orderBeingProcessedList) { if
	 * (next.fileOrderId.contentEquals(fileOrderIdToBeRemoved)) {
	 * next.orderExeReportReceived = true; guiWindow.AddText(" Order '" +
	 * fileOrderIdToBeRemoved + "' found :orderExeReportReceived = true" ); break; }
	 * }
	 * 
	 * guiWindow.AddText("	After tag:");
	 * displayOrderBeingProcessedList(orderBeingProcessedList, guiWindow);
	 * 
	 * } // ********************************************************** // ***
	 * updateOrderList: update the orderList when receiving a Contract Info Report
	 * new broadcast // **********************************************************
	 * public void updateOrderList(List<Order> orderList, String
	 * orderExeReportOrderId, String orderExeReportClientOrderId, String
	 * orderExeReportRevisionNo, String orderState, String orderAction, GUIWindow
	 * guiWindow) { boolean orderFound = false;
	 * 
	 * guiWindow.AddText("*** start of updateOrderList"+" OrderId = " +
	 * orderExeReportOrderId + " ClientOrderId= " + orderExeReportClientOrderId +
	 * " RevisionNo= "+ orderExeReportRevisionNo + " orderState = " + orderState +
	 * " orderAction = " + orderAction ); // displayOrderList( orderList,
	 * guiWindow); for (Order next : orderList) { // update orders in Order List for
	 * the given file order ID that have not been sent to M7 if
	 * (next.fileOrderId.contentEquals(orderExeReportClientOrderId) ) { orderFound =
	 * true; // only update the order Id in Order List if higher than the previous
	 * one // indeed in case of an order update with a priority change (price
	 * change, qty increase) // the order is 1st deleted then Added, but M7 sends
	 * the ADD first and the DEL (with a lower revision No) afterwards // we keep
	 * the highest order Id that corresponds to the new order (ADD) (and not
	 * overwrite it with the DEL info) if (next.m7OrderId.contentEquals("")) {
	 * next.m7OrderId = orderExeReportOrderId; next.m7RevisionNo =
	 * orderExeReportRevisionNo; next.state = orderState; next.action = orderAction;
	 * } else if (Integer.valueOf(orderExeReportOrderId) >=
	 * Integer.valueOf(next.m7OrderId) ) { next.m7OrderId = orderExeReportOrderId;
	 * next.m7RevisionNo = orderExeReportRevisionNo; next.state = orderState;
	 * next.action = orderAction; } else
	 * guiWindow.AddText("  no order list update since " + orderExeReportOrderId +
	 * "<" + next.m7OrderId);
	 * 
	 * //guiWindow.AddText("  File order ID found : order List updated: " +
	 * next.Display()); }
	 * 
	 * } // the whole list has been checked if (orderFound == false){
	 * guiWindow.AddText("  orderExeReportOrderId " + orderExeReportOrderId +
	 * " NOT Found in the whole Order List"); } displayOrderList(orderList,
	 * guiWindow); } // **********************************************************
	 * // *** displayOrderList //
	 * ********************************************************** public void
	 * displayOrderList(List<Order> orderList, GUIWindow guiWindow) { for (Order
	 * next : orderList) { guiWindow.AddText("displayOrderList :'" +
	 * next.Display()+"'");
	 * 
	 * } } // ********************************************************** // ***
	 * displayOrderBeingProcessedList //
	 * ********************************************************** public void
	 * displayOrderBeingProcessedList(List<OrderBeingProcessed>
	 * orderBeingProcessedList, GUIWindow guiWindow) { for (OrderBeingProcessed next
	 * : orderBeingProcessedList) {
	 * guiWindow.AddText("displayOrderBeingProcessedList :'" + next.Display()+"'");
	 * } }
	 */

}
	
	
