import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JOptionPane;

public class BG_Member_List {
	private CopyOnWriteArrayList<BG_Member> bgMemberList = new CopyOnWriteArrayList<>();
	private int nbBG = 0;
	private ControlBoard controlBoardFrame;
	MembersList memberList;
	
	// constructor
	public BG_Member_List(ControlBoard controlBoardFrame, MembersList memberList) {
		this.controlBoardFrame = controlBoardFrame;
		this.memberList = memberList;
	}

	public int getNbBG() {
		return nbBG;
	}
	public void setNbBG(int nbBG) {
		this.nbBG = nbBG;
	}
	private final static String FILE_BG_MEMBER_NAME = "BG-Member List.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_BG_MEMBER_LIST_HEADER = 	
														"Account/BG Id"+CSV_SEPARATOR 
														+ "BG Name"+CSV_SEPARATOR 													
														+ "Member ID"+CSV_SEPARATOR
														+ "Member Name" + CSV_SEPARATOR
														+ "OTC" + CSV_SEPARATOR
														+ "State" + CSV_SEPARATOR
														+ "Assigned Areas" + CSV_SEPARATOR
														+ "Assigned Products"
														+ NEW_LINE_SEPARATOR;
	// ***************************************************
	// *** WriteBGMemberListCSVFile
	// ***************************************************	
	public void WriteBGMemberListCSVFile() {
		try {
			FileWriter fileWriter = new FileWriter(FILE_BG_MEMBER_NAME);
			fileWriter.append(FILE_BG_MEMBER_LIST_HEADER);
			
			for (BG_Member next : bgMemberList) {
				String row = 	next.BG_AccountID + CSV_SEPARATOR
								+ next.BG_Name + CSV_SEPARATOR		
								+ next.MemberID + CSV_SEPARATOR
								+ next.MemberName + CSV_SEPARATOR
								+ next.preArranged_OTC + CSV_SEPARATOR
								+ next.state + CSV_SEPARATOR
								+ next.assigned_DeliveryAreas.toString() + CSV_SEPARATOR
								+ next.assigned_Products.toString()
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				
			}			
			fileWriter.flush();
			fileWriter.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_BG_MEMBER_NAME + " file cannot be accessed : please check it is well closed.";
			JOptionPane.showMessageDialog(controlBoardFrame, msg);
		}		
	}
	// ***************************************************
	// *** Add a BG Member couple to the list 
	// ***************************************************
	public int AddBGMember(BG_Member bgMember, Log appLog) {

		bgMember.MemberName  = memberList.ReturnMemberInfo(bgMember.MemberID, appLog).getMemberName();
		bgMemberList.add(bgMember);
		this.setNbBG(this.getNbBG() + 1);
		return this.getNbBG();
	}
	// ************************************************************************
	// *** Update a BG in the list: add or update if BG already listed 
	// ************************************************************************
	public boolean UpdateBGMemberList(BG_Member BG_Member, MembersList memberList, Log appLog) {

		boolean BG_Found = false;

		for (BG_Member next : bgMemberList) {
	
			if ( next.BG_AccountID.contentEquals(BG_Member.BG_AccountID)) {
				// BG already listed : just update values
				BG_Found = true;
				next.MemberID = BG_Member.MemberID;
				next.MemberName = BG_Member.MemberName;
				break;
			}
			if (BG_Found == false) {
				this.AddBGMember(BG_Member, appLog);
			}			
		}
		return BG_Found;
	}
	// ************************************************************************
	// *** Update a BG in the list: add or update if BG already listed 
	// ************************************************************************
	public BG_Member ReturnBGMemberInfoList(String BG, Log appLog) {

		BG_Member bgMemberToReturn = new BG_Member();
		boolean BG_Found = false;
		appLog.logger.info("ReturnBGMemberInfoList Searching for BG = "+BG + " list size = "+ this.getNbBG());

		for (BG_Member next : this.bgMemberList) {
			
			if ( next.BG_AccountID.contentEquals(BG)) {
				// get Member info for that BG
				BG_Found = true;
				bgMemberToReturn.BG_AccountID = BG;
				bgMemberToReturn.MemberID = next.MemberID;
				bgMemberToReturn.MemberName = next.MemberName;
				appLog.logger.info("BG found = "+BG + " Member ID = "+ next.MemberID);
				break;
			}
		}
		if (BG_Found == false) {
			bgMemberToReturn.MemberID= "NOT_FOUND";
			bgMemberToReturn.MemberName= "...";
			appLog.logger.info("BG Not found = "+BG );
		}
		return bgMemberToReturn;
	}
}
